/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.tracing;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FilePermission;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Date;

import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.init.InitializerIFace;

public class DefaultLogger implements IFaceLogger, InitializerIFace {
    private static final String EMPTY_STRING = "                    ";
    private static final String TRACE_FILE_PATH = "/tmp/";
    private static final String TRACE_FILE_NAME = "af-";
    private String _name = null;
    private SourceBean _config = null;
    private boolean _isEnabled = false;
    private boolean _isOutOfService = false;
    private PrintWriter _traceFile = null;
    private String _lastTracedDate = null;
    private int _minLogSeverity = 0;
	private boolean _debug = false;
	private boolean _traceThreadName = false;

    public DefaultLogger() {
        super();
        _name = null;
        _config = null;
        _isEnabled = false;
        _isOutOfService = false;
        _traceFile = null;
        _lastTracedDate = null;
        _minLogSeverity = 0;
        _debug = true;
    } // public DefaultLogger()

    public String getName() {
        return _name;
    } // public String getName()

    public void setName(String name) {
        _name = name;
    } // public void setName(String name)

    public synchronized void init(SourceBean config) {
        _config = config;
    } //public synchronized void init(SourceBean config)

    public synchronized SourceBean getConfig() {
        return _config;
    } //public synchronized SourceBean getConfig()

    /**
     * Abilita il tracciamento.
     */
    public synchronized void enable() {
        if (_isEnabled)
            return;
        try {
            openTraceFile();
        } // try
        catch (Exception ex) {
            System.out.println(
                "DefaultLogger::enable: [" + _name + "] " + ex.getMessage());
            ex.printStackTrace();
        } // catch(Exception ex)
        _isEnabled = true;
    } // public synchronized void enable()

    /**
     * Disabilita il tracciamento
     */
    public synchronized void disable() {
        if (!_isEnabled)
            return;
        try {
            closeTraceFile();
        } // try
        catch (Exception ex) {
            System.out.println(
                "DefaultLogger::disable: [" + _name + "] " + ex.getMessage());
            ex.printStackTrace();
        } // catch(Exception ex)
        _isEnabled = false;
    } // public synchronized void disable()

    /**
     * Verifica se il tracer &egrave; abilitato a tracciare.
     * @return <code>boolean</code>
     */
    public synchronized boolean isEnabled() {
        return _isEnabled;
    } // public synchronized boolean isEnabled()

    /**
     * Questo metodo verifica se il parametro severity e ritorna:
     * <em>false</em> se il severity non  <em>DEBUG</em> e inferiore della severity minima.
     * <em>true</em>se il severity   <em>DEBUG</em> o maggiore della severity minima.
     * @param severity codice della severity
     * @return il risultato della comparazione.
     */
    public synchronized boolean testToTrace(int severity) {
        if (!_isEnabled)
            return false;
        if (severity == TracerSingleton.DEBUG)
            return _debug;
        return (severity < _minLogSeverity) ? false : true;
    } // public synchronized boolean testToTrace(int severity)

    /**
     * Questo metodo effettua la scrittura nel file di log.
     * @param application la stringa che descrive il punto nel quale  avvenuta la
     * chiamata per il tracciamento.
     * @param severity codice della severity.
     * @param message descrizione del messaggio.
     */
    public synchronized void trace(
        String application,
        int severity,
        String message) {
        if (!testToTrace(severity))
            return;
        String currentTimestamp = getTimestamp();
        application = application.concat(EMPTY_STRING);
        application = application.substring(0, 15);
        String fullMessage =
            currentTimestamp
                + " "
                + getThreadName()
                + " "
                + application
                + " "
                + getSeverity(severity)
                + " "
                + message;
        if (_isEnabled) {
            testToSwitch();
            if (!_isOutOfService) {
                try {
                    _traceFile.println(fullMessage);
                    _traceFile.flush();
                } // try
                catch (Exception ex) {
                    _isOutOfService = true;
                } // catch (Exception ex) try
            } // if (!_isOutOfService)
            else
                System.out.println(fullMessage);
        } // if (_isEnabled)
        else
            System.out.println(fullMessage);
    } // private synchronized void trace(String application, int severity, String message)

    /**
     * Ritorna la data odierna nel formato timestamp.
     * @return <code>String</code> rappresentante il timestamp.
     */
    private String getTimestamp() {
        Date date = new Date();
        SimpleDateFormat formatter = new SimpleDateFormat("yyMMdd hh:mm:ss");
        return formatter.format(date);
    } // private String getTimestamp()

    /**
     * Ritorna il nome del Thread che verr utilizzato nel tracciamento.
     * @return <code>String</code> il nome del Thread.
     */
    private String getThreadName() {
        return _traceThreadName ? Thread.currentThread().getName() : "";
    } // private String getThreadName()

    /**
     * Ritorna la descrizione della severity associata ad un codice.
     * @return <code>String</code> la descrizione della severity.
     * @param severity codice della severity.
     */
    private String getSeverity(int severity) {
        String stringSeverity = null;
        switch (severity) {
            case TracerSingleton.INFORMATION :
                stringSeverity = "INFORMATION";
                break;
            case TracerSingleton.WARNING :
                stringSeverity = "WARNING";
                break;
            case TracerSingleton.MINOR :
                stringSeverity = "MINOR";
                break;
            case TracerSingleton.MAJOR :
                stringSeverity = "MAJOR";
                break;
            case TracerSingleton.CRITICAL :
                stringSeverity = "CRITICAL";
                break;
            case TracerSingleton.DEBUG :
                stringSeverity = "DEBUG";
                break;
        } // switch (severity)
        stringSeverity = stringSeverity.concat(EMPTY_STRING);
        return stringSeverity.substring(0, 12);
    } // private String getSeverity(int severity)

    private void testToSwitch() {
        if (!_isEnabled)
            return;
        String currentTimestamp = getTimestamp();
        String currentTracedDate = currentTimestamp.substring(0, 6);
        if (_lastTracedDate.equalsIgnoreCase(currentTracedDate))
            return;
        closeTraceFile();
        openTraceFile();
    } // private void testToSwitch()

    /**
     * Questo metodo crea il file di log recuperando le informazioni dal file XML.
     */
    private void openTraceFile() {
        String traceFilePath = (String) getConfig().getAttribute("TRACE_PATH");
        if (traceFilePath == null)
            traceFilePath = TRACE_FILE_PATH;
        String currentTimestamp = getTimestamp();
        _lastTracedDate = currentTimestamp.substring(0, 6);
        String traceFileName = (String) getConfig().getAttribute("TRACE_NAME");
        if (traceFileName == null)
            traceFileName = TRACE_FILE_NAME;
        //livello dei tracciamenti definito per ciascun logger nella busta config  
        String minLogSeverity =
            (String) getConfig().getAttribute("TRACE_MIN_LOG_SEVERITY");
        _minLogSeverity = 0;
        try {
            _minLogSeverity = Integer.parseInt(minLogSeverity);
            if (_minLogSeverity < 0)
                _minLogSeverity = 0;
        } // try
        catch (Exception ex) {
            System.out.println(
                "DefaultLogger::openTraceFile: ["
                    + _name
                    + "] parametro TRACE_MIN_LOG_SEVERITY non valido");
            ex.printStackTrace();
        } // catch (Exception ex)
        String debug = (String) getConfig().getAttribute("DEBUG");
        _debug = ((debug != null) && (debug.equalsIgnoreCase("TRUE")));
        
		String traceThreadName = (String) getConfig().getAttribute("TRACE_THREAD_NAME");
		_traceThreadName = ((traceThreadName != null) && (traceThreadName.equalsIgnoreCase("TRUE")));
		
        String appendStr = (String) getConfig().getAttribute("APPEND");
        boolean append =
            ((appendStr != null) && (appendStr.equalsIgnoreCase("TRUE")));
        String fileName = "";
        if (ConfigSingleton.getRootPath() != null)
            fileName += ConfigSingleton.getRootPath();
        fileName += traceFilePath + traceFileName + _lastTracedDate + ".log";
        File messageFile = new File(fileName);
        FilePermission permission =
            new FilePermission(traceFilePath, "read, write");
        try {
            if (append)
                _traceFile =
                    new PrintWriter(
                        new BufferedWriter(new FileWriter(fileName, true)));
            else
                _traceFile =
                    new PrintWriter(
                        new BufferedWriter(new FileWriter(fileName, false)));
            _isOutOfService = false;
        } // try
        catch (IOException ex) {
            _isOutOfService = true;
            System.out.println(
                "DefaultLogger::openTraceFile: ["
                    + _name
                    + "]  impossibile aprire il file di log");
        } // catch (IOException ex)
    } // private void openTraceFile()

    /**
     * Questo metodo chiude  il file di log .
     */
    private void closeTraceFile() {
        try {
            _traceFile.close();
            _isOutOfService = false;
        } // try
        catch (Exception ex) {
            _isOutOfService = true;
            System.out.println(
                "DefaultLogger::openTraceFile: ["
                    + _name
                    + "] impossibile chiudere il file di log");
        } // catch (Exception ex)
    } // private void closeTraceFile()    
} // public class DefaultLogger implements IFaceLogger
