/**
 * 
 * Copyright 2004, 2005 Engineering Ingegneria Informatica S.p.A. & Sinapsi
 * S.p.A.
 * 
 * This file is part of Spago.
 * 
 * Spago is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * Spago is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Spago; if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 *  
 */

package it.eng.spago.validation.coordinator;

import it.eng.spago.base.Constants;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.validation.ValidationEngineIFace;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

/**
 * 
 * Coordinator of the validation process This class retrieves from the
 * configuration the name of the class that implements the server side
 * validation, instantiates it and invokes it during the validation phase
 */
public class ValidationCoordinator {

    private static Class engineClass = null;

    /**
     * Performs the validation of the request
     * 
     * @param businessType - Type of service (action/page)
     * @param businessName - Logical name of the service
     * @param requestContainer
     * @param responseContainer
     * @return true if the process can be invoked
     */
    public static boolean validate(final String businessType, final String businessName,
    		RequestContextIFace requestContext) {
        ValidationEngineIFace engine = getValidationEngine(requestContext);
        
        if (engine != null) {
            if (!engine.validate(businessType, businessName, requestContext))
                // Some errors was produced by validation: execute the service only if required
                return (!engine.isBlocking());
            else
                // No errors produced: execute the service
                return true;
        }
        else
            // No validation engine: execute the service
            return true;
    }

    private static ValidationEngineIFace getValidationEngine(RequestContextIFace requestContext) {

        ValidationEngineIFace result = null;
        String engineClassName = null;

        if (engineClass == null) {
            synchronized (ValidationCoordinator.class) {
                ConfigSingleton config = ConfigSingleton.getInstance();
                engineClassName = (String) config.getAttribute("VALIDATORS.ENGINE.CLASS");

                if (engineClassName != null) {
                    try {
                        engineClass = Class.forName(engineClassName);
                    } catch (ClassNotFoundException ex) {
                        TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                                "ValidationCoordinator::getValidationEngine:", ex);
                        return null;
                    }
                }
            }// synchronized (ValidatorLocator.class)
        }

        if (engineClass != null) {

            try {
                Constructor validatorConstructor = engineClass.getConstructor(new Class[0]);
                result = (ValidationEngineIFace) validatorConstructor.newInstance(new Object[0]);
            } catch (NoSuchMethodException ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                        "ValidationCoordinator::getValidationEngine:", ex);
                return null;
            } catch (InstantiationException ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                        "ValidationCoordinator::getValidationEngine:", ex);
                return null;
            } catch (InvocationTargetException ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                        "ValidationCoordinator::getValidationEngine:", ex);
                return null;
            } catch (IllegalAccessException ex) {
                TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.CRITICAL,
                        "ValidationCoordinator::getValidationEngine:", ex);
                return null;
            }
        } else {
            TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.WARNING,
                    "ValidationCoordinator::getValidationEngine: Validation engine not specified");
        }

        return result;
    }

}
