/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.validation.fieldvalidators;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.dispatching.service.DefaultRequestContext;
import it.eng.spago.message.MessageBundle;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.validation.EMFValidationError;
import it.eng.spago.validation.FieldValidatorIFace;
import it.eng.spago.validation.impl.ValidationServices;

import java.util.Map;

/**
 * @author zoppello
 *
 * An abstract class that implements FieldValidatorIFace, in this class there is the
 * code for handling of the TYPED_SERVICE_REQUEST source bean that will contains the field with
 * alias after the validation
 */
public abstract class AbstractFieldValidator extends DefaultRequestContext implements FieldValidatorIFace {

    public static final String GENERIC_ERROR_IDENTIFIER = "ERROR_CODE";
    public static final String DEFAULT_VALUE_IDENTIFIER = "DEFAULT_VALUE";
	
	/**
	 * The configuration of the field validator, it will contain the contente defined
	 * in <CONFIG/> section of each validator
	 */
	private SourceBean config = null;
	
	/**
	 * Save in the Typed Service Request the attribute with name and value
	 * 
	 * Subclasses must use this method to save the field converted with typed value and alias
	 * 
	 * @param serviceRequest The Spago Service Request 
	 * @param name The name of the attribute to store
	 * @param value The value to store 
	 */
	public void saveTypedValue(SourceBean serviceRequest, String name, Object value){
		try{
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.DEBUG, "AbstractFieldValidator::saveTypedValue: name ["+name + "] value ["+value+"]");
			ValidationServices.getTypedServiceRequest(serviceRequest).setAttribute(name, value);
			
		}catch (Exception e) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL, "AbstractFieldValidator::saveTypedValue:", e);
		}
	}
	
	/**
	 * The getter method for the configuration of the field Validator
	 * 
	 * @see it.eng.spago.validation.FieldValidatorIFace#getConfig()
	 */
	public SourceBean getConfig() {
		return this.config;
	}

	/**
	 * Initialize The validator with his configuration 
	 * 
	 * @see it.eng.spago.validation.FieldValidatorIFace#init(it.eng.spago.base.SourceBean)
	 */
	public void init(SourceBean config) {
		this.config = config;	
	}

    /**
     * Retrieve a parameter from the field configuration (if present) or from the validator configuration
     * @return The value for this parameter.
     */
    public String getParameter(final String paramName, final String defaultValue, final Map fieldValidationParams) {
        
        String errorCode = (String)fieldValidationParams.get(paramName);
        if (errorCode == null) {
            SourceBean config = getConfig();
            if (config != null){
                errorCode = (String)config.getAttribute(paramName);
            }
            
            if (errorCode == null) {
                errorCode = defaultValue;
            }
        }
        
        return errorCode;
    }
    
    /**
     * Return the field name used in error messages.
     * @param fieldName Name of the field in the form.
     * @return Field name used in error messages.
     */
    public String getUserFieldName(final String fieldName) {
        return MessageBundle.getServiceMessage(fieldName, getResponseContainer());
    }

    /* (non-Javadoc)
     * @see it.eng.spago.validation.FieldValidatorIFace#setDefault(it.eng.spago.base.SourceBean, java.lang.String, java.util.Map)
     */
    public boolean setDefault(SourceBean serviceRequest, String fieldName, Map fieldValidationParams) throws EMFValidationError {
        boolean result = false;
    	String defaultValue = (String)fieldValidationParams.get(DEFAULT_VALUE_IDENTIFIER);
        
        if (defaultValue == null) {
            SourceBean config = getConfig();
            if (config != null){
                defaultValue = (String)config.getAttribute(DEFAULT_VALUE_IDENTIFIER);
            }
        }
        
        if (defaultValue == null) {
            return result;
        }
        
        // Imposto l'attributo non tipizzato
        try {
            serviceRequest.updAttribute(fieldName, defaultValue);
            result = true;
        }catch(SourceBeanException eX){
            TracerSingleton.log(
                    Constants.NOME_MODULO,
                    TracerSingleton.WARNING,
                    "AbstractFieldValidator::setDefault: valore non valido");
        }
        
        // Imposto l'attributo tipizzato
        validateField(serviceRequest, fieldName, defaultValue, fieldValidationParams);
        return result;
    }
    
}
