/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.validation.fieldvalidators;

import it.eng.spago.base.SourceBean;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.validation.EMFValidationError;
import it.eng.spago.validation.impl.SyntaxChecker;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.validator.GenericValidator;

/**
 * @author zoppello
 *
 * This class checks if a field is a valid italian fiscal code or partita iva string
 * 
 * This Validator doesn't apply type conversion, if it succed the original value will be put in TYPED_SERVICE_REQUEST
 * with an alias if this is configured in validation.xml file
 */
public class FiscalCodeValidator extends AbstractFieldValidator {

    private static final long serialVersionUID = 1L;
    public static final String ERROR_CF_IDENTIFIER = "ERROR_CF_CODE";
    public static final String ERROR_PI_IDENTIFIER = "ERROR_PI_CODE";

    public static final String ERRORE_CAMPO_CODICE_FISCALE = "10101";
	public static final String ERRORE_CAMPO_PARTITA_IVA = "10102";
    
	private static final String CODICE_FISCALE_REGEXP =
		"^([A-Z]{6}[A-Z\\d]{2}[A-Z][A-Z\\d]{2}[A-Z][A-Z\\d]{3}[A-Z])*$";
	
	// Espressioni regolari per la gestione dell'attributo "regExp"
	private static final String DIGIT_REGEXP = "^([0-9])*$";
    
	/** 
	 * Implements the logic described in JavaDoc of the class
	 * 
	 * @see it.eng.spago.validation.FieldValidatorIFace#validateField(it.eng.spago.base.SourceBean, java.lang.String, java.lang.String, java.util.Map)
	 */
	public void validateField(SourceBean serviceRequest, String fieldName, String value, Map fieldValidationParams) throws EMFValidationError {
		char primoChar = value.charAt(0);
		// Verifico se  un codice fiscale o una partita iva
		List params = new ArrayList(2);
        params.add(fieldName);
        params.add(getUserFieldName(fieldName));
		if(GenericValidator.matchRegexp(String.valueOf(primoChar), DIGIT_REGEXP))
		{
			if(gestionePI(value,11)==null)
				throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName, getParameter(ERROR_PI_IDENTIFIER, ERRORE_CAMPO_PARTITA_IVA, fieldValidationParams), params);
		}		
		else
		{
			if (!controllaCF(value))
				throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName, getParameter(ERROR_CF_IDENTIFIER, ERRORE_CAMPO_CODICE_FISCALE, fieldValidationParams), params);
		}	
		
		
//		 This Field Validator Class doesn't apply type conversion it apply only alias on field 
		String fieldAlias = (String)fieldValidationParams.get(SyntaxChecker.PARAM_FIELD_ALIAS);
		if (fieldAlias == null){
			fieldAlias = fieldName;
		}
		
		saveTypedValue(serviceRequest, fieldAlias, value);
	}
	
	

	/**
	 * Verifica il formato del Codice Fiscale
	 */
	private static boolean controllaCF(String cf) {
		cf = cf.toUpperCase().trim();
		return (GenericValidator.matchRegexp(cf, CODICE_FISCALE_REGEXP));
	} //end controllaCF
	
	
	/**
	 * Verifica il formato della Partita IVA.
	 * Ritorna la Partita IVA completata di zeri
	 * eventualmente.
	 */
	private static String gestionePI(String pi, int lng) {
		int valueLng = pi.length();
		if (valueLng <= lng) {
			if(!GenericValidator.matchRegexp(pi, DIGIT_REGEXP))
				return null;
			pi = riempiZero(pi, lng);
			if (carattereControlloCorretto(pi)) return pi;
			return null;
		} else
			return null;
	} //end gestionePI
	
	private static boolean carattereControlloCorretto(String pi)
	{
		int i, c, s;
		s = 0;
		for (i = 0; i <= 9; i += 2)
		{
			s += pi.charAt(i) - '0';
		}
		for (i = 1; i <= 9; i += 2) 
		{
			c = 2 * (pi.charAt(i) - '0');
			if (c > 9)
				c = c - 9;
			s += c;
		}
		if ((10 - s % 10) % 10 != pi.charAt(10) - '0')
		{
			return false;
		}
		return true;
	}
	
	/**
	 * Aggiunge a sinistra della stringa di input un numero 
	 * di zeri pari alla lunghezza specificata meno la 
	 * lunghezza della stringa. 
	 */
	public static String riempiZero(String value, int lng) {
		int lung = lng - value.length();
		for (int i = 0; i < lung; i++)
			 value = "0" + value;
		return value;	 
	} //end riempiZero

}
