/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.validation.fieldvalidators;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.error.EMFErrorSeverity;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.validation.EMFValidationError;
import it.eng.spago.validation.impl.SyntaxChecker;

import java.text.DecimalFormatSymbols;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.validator.GenericValidator;
/**
 * @author zoppello
 *
 * This class checks if a field is a valid number.
 * 
 * This implementation need that this validator should be configured with parameters regarding language
 * and country, with this information the notion of "decimal separator" will depend on Locale object
 * istantiate using this information
 * 
 * For Example:
 * 
 * <FIELD-VALIDATOR fieldType="NUMERIC" fieldValidatorClass="it.eng.spago.validation.fieldvalidators.NumericValidator">
 *		<CONFIG>
 *			<LANGUAGE language="it"/>
 *			<COUNTRY country="IT"/>
 *		</CONFIG>
 *	</FIELD-VALIDATOR>
 *
 *	If the config section the Italian Locale will be used
 *
 *  This Validator will convert the original value in a java.lang.Double object if it succeed and  
 *  this will be putted  in TYPED_SERVICE_REQUEST with an alias if this is configured in validation.xml file
 *  
 */
public class NumericValidator extends AbstractFieldValidator {
    
    private static final long serialVersionUID = 1L;
    public static final String ERROR_MAXVALUE_IDENTIFIER = "ERROR_CODE_MAXVALUE";
    public static final String ERROR_MINVALUE_IDENTIFIER = "ERROR_CODE_MINVALUE";
    public static final String ERROR_MIN_MAX_VALUE_IDENTIFIER = "ERROR_CODE_MIN_MAXVALUE";

	public static final String ERRORE_CAMPO_NUMERICO = "10106"; 
	public static final String ERRORE_MAX_VALUE = "10118";
    public static final String ERRORE_MIN_VALUE = "10119";
    public static final String ERRORE_MIN_MAX_VALUE = "10120";
    
    private Locale locale = null;
    private NumberFormat numberFormat = null;
    private String fieldAlias = null;
    private List messagesParams = null;
    
	public void init(SourceBean config) {
        super.init(config);

        String language = "it";
        String country = "IT";
        
        if (config != null){
            language = (String)config.getAttribute("LANGUAGE.language");
            country = (String)config.getAttribute("COUNTRY.country");
        }
        
        // Parameter list for error messages
        messagesParams = new ArrayList();

        // Create all the objects locale dependent
        locale = new Locale(language, country);
        numberFormat = NumberFormat.getNumberInstance(locale);
	}
    
    /**
     * @return The Locale built from configuration information.
     */
    public Locale getLocale() {
        return locale;
    }
    
    /**
     * @return The NumberFormat built according to the locale created from configuration information.
     */
    public NumberFormat getNumberFormat() {
        return numberFormat;
    }
    
    public String getFieldAlias() {
        return fieldAlias;
    }

    public void setFieldAlias(String fieldAlias) {
        this.fieldAlias = fieldAlias;
    }

    public List getMessagesParams() {
        return messagesParams;
    }

    public void setMessagesParams(List messagesParams) {
        this.messagesParams = messagesParams;
    }

    /** 
	 * Implements the logic described in JavaDoc of the class
	 * 
	 * @see it.eng.spago.validation.FieldValidatorIFace#validateField(it.eng.spago.base.SourceBean, java.lang.String, java.lang.String, java.util.Map)
	 */
	public void validateField(SourceBean serviceRequest, String fieldName, String value, Map fieldValidationParams) throws EMFValidationError {
		DecimalFormatSymbols decimalFormatSymbols = new DecimalFormatSymbols(getLocale());	
		char decimalSeparator = decimalFormatSymbols.getDecimalSeparator();
		String regexp = "^[-+]?\\d*\\"+decimalSeparator+"?\\d*$";
		
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "NumericValidator::validateField");
		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.DEBUG, "NumericValidator::validateField: Validate ["+value+"] with regexp ["+ regexp + "]");
		
		List params = getMessagesParams();
		params.add(fieldName);
        params.add(getUserFieldName(fieldName));
		if (!GenericValidator.matchRegexp(value, regexp)){
			throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName,getParameter(GENERIC_ERROR_IDENTIFIER, ERRORE_CAMPO_NUMERICO, fieldValidationParams),params);
		}

        // Handle optional checks on the decimal part of the number
        handleDecimals(serviceRequest, fieldName, value, fieldValidationParams, decimalSeparator, params);
        
		//This Field Validator Class doesn't apply type conversion it apply only alias on field 
		String alias = getParameter(SyntaxChecker.PARAM_FIELD_ALIAS, null, fieldValidationParams);
		if (alias == null){
			alias = fieldName;
		}
        setFieldAlias(alias);
		
		if (value.startsWith("+")){
			value = value.substring(1);
		}
		
		String maxValue = getParameter(SyntaxChecker.MAX_VALUE_ATTRIBUTE, null, fieldValidationParams);
		String minValue = getParameter(SyntaxChecker.MIN_VALUE_ATTRIBUTE, null, fieldValidationParams);
		
		Double convertedValue  = null;
		Double convertedMaxValue  = null;
		Double convertedMinValue  = null;
		
		try {
	        Number number = numberFormat.parse(value);
	        convertedValue = new Double(number.doubleValue());
	        
	        if (!GenericValidator.isBlankOrNull(maxValue)){
	        	Number maxValueNumber = numberFormat.parse(maxValue);
	        	convertedMaxValue = new Double(maxValueNumber.doubleValue());
	        }
	        
	        if (!GenericValidator.isBlankOrNull(minValue)){
	        	Number minValueNumber = numberFormat.parse(minValue);
	        	convertedMinValue = new Double(minValueNumber.doubleValue());
	        }
            
            // If both MIN_VALUE and MAX_VALUE are specified, and the number is not in the range, a message
            // error reporting the range has to be produced
            if ((convertedMinValue != null) && (convertedMaxValue != null)
                 && ((convertedValue.doubleValue() > convertedMaxValue.doubleValue()) ||
                     (convertedValue.doubleValue() < convertedMinValue.doubleValue()))) {
                params.add(minValue);
                params.add(maxValue);
                throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName, getParameter(ERROR_MIN_MAX_VALUE_IDENTIFIER, ERRORE_MIN_MAX_VALUE, fieldValidationParams), params);
            }
            
            // Check if only MAX_VALUE has been specified and if the number is greater
            if ((convertedMaxValue != null)
	         && (convertedValue.doubleValue() > convertedMaxValue.doubleValue())){
		
				params.add(maxValue);	
				throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName, getParameter(ERROR_MAXVALUE_IDENTIFIER, ERRORE_MAX_VALUE, fieldValidationParams), params);

			}	
			
            // Check if only MIN_VALUE has been specified and if the number is smaller
	        if ((convertedMinValue != null) 
	            &&  (convertedValue.doubleValue() < convertedMinValue.doubleValue())){
			
    			params.add(minValue);	
    			throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName, getParameter(ERROR_MINVALUE_IDENTIFIER, ERRORE_MIN_VALUE, fieldValidationParams), params);

			}

	        saveTypedValue(serviceRequest, alias, convertedValue);
	    } catch (ParseException e) {
	    	throw new EMFValidationError(EMFErrorSeverity.ERROR, fieldName, getParameter(GENERIC_ERROR_IDENTIFIER, ERRORE_CAMPO_NUMERICO, fieldValidationParams),params);
	    }
	}
    
    /**
     * Handle optional checks on the decimal part of the number
     * @see it.eng.spago.validation.FieldValidatorIFace#validateField(it.eng.spago.base.SourceBean, java.lang.String, java.lang.String, java.util.Map)
     */
    public void handleDecimals(SourceBean serviceRequest, String fieldName, String value, Map fieldValidationParams, char decimalSeparator, List params) throws EMFValidationError {
    }
	
}
