/**

 Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

 This file is part of Spago.

 Spago is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation; either version 2.1 of the License, or
 any later version.

 Spago is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with Spago; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago.validation.impl;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanAttribute;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.dispatching.service.RequestContextIFace;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.util.ContextScooping;
import it.eng.spago.util.StringUtils;
import it.eng.spago.validation.ValidationEngineIFace;
import it.eng.spago.validation.ValidatorIFace;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.jamonapi.Monitor;
import com.jamonapi.MonitorFactory;

/**
 * 
 * @author boccalon
 * 
 * Questo modulo permette di validare (controllare) i campi di una richiesta
 * tramite validatori diversi. L'attivit di questo modulo  controllata
 * congiuntamente dai file di configurazione validation.xml e validators.xml.
 * 
 * Per ogni servizio andr configurata una busta di tipo SERVICE in cui
 * l'attributo "name"  il nome della corrispondente pagina o action.
 * 
 * Questo modulo, inoltre effettua anche elaborazioni aggiuntive sui dati di
 * input, controllate dai seguenti parametri: - trim="true | false" : se
 * impostato a true elimina gli spazi iniziali e finali del campo. -
 * toUpperCase="true | false" : se impostato a true trasforma il campo in
 * maiuscolo. Ecco un esempio:
 * 
 * <SERVICE name="GESTIONE_ANAGRAFICA_AGENZIE" type="ACTION"> <FIELD
 * name="INCODAGE" trim="true" toUppercase="true" /> <FIELD name="INPARIVA"
 * trim="true" /> </SERVICE>
 * 
 * Il file validators.xml indica quali classi di validazione devono essere
 * applicate alle varie sezioni. Ecco un esempio di configurazione: <VALIDATORS>
 * <VALIDATOR name="ANDATA" class="com.sample.AndataValidator" /> <VALIDATOR
 * name="RITORNO" class="com.sample.RitornoValidator" /> <VALIDATOR
 * name="AUTO_E_COSE" class="com.sample.AutoCoseValidator" /> </VALIDATORS> Ogni
 * validatore ha un nome logico e una classe di implementazione.
 * 
 */

public class ValidationImpl implements ValidationEngineIFace {

	private boolean blocking = true;

	public static final String NAME_ATTRIBUTE = "NAME";

	public static final String TOUPPERCASE_ATTRIBUTE = "TOUPPERCASE";

	public static final String TRIM_ATTRIBUTE = "TRIM";

	public static final String TYPE_ATTRIBUTE = "TYPE";
	
	public static final String ASK_CONFIRM = "ASK_CONFIRM";

	public static final String SKIP_CONFIRM = "SKIP_CONFIRM";

	public static final String LIST_SYMBOL = "@";

	/**
	 * Default constructor
	 */
	public ValidationImpl() {
	}

	/**
	 * Return the validation enveloped related to the service with name business name and business service type
	 * @param businessName The business name of the service
	 * @param businessType The business type of the service
	 * @return
	 */
	public SourceBean getServiceValidationSourceBean( final String businessName, final String businessType) {
		
		SourceBean serviceValidations = null;
		ConfigSingleton config = ConfigSingleton.getInstance();
		// Search in the configuration the validation section 
		Object objServices = config.getFilteredSourceBeanAttribute(
				ValidatorLocator.VALIDATION_CONFIG_HEADER,
				ValidatorLocator.VALIDATION_SERVICE_NAME, businessName);
		if (objServices instanceof SourceBean)
			// Ok exact match of on section
			serviceValidations = (SourceBean) objServices;
		else if (objServices instanceof List) {
			// This case if there are two services with the same name but with different types
			// in the validations.xml 
			// In that case we must decide on the service type attribute
			for (int i = 0; i < ((List) objServices).size(); i++) {
				SourceBean tmp = (SourceBean) (((List) objServices).get(i));
				if (((String) tmp.getAttribute("TYPE"))
						.equalsIgnoreCase(businessType))
					serviceValidations = tmp;
				break;
			}
		}
		return serviceValidations;
	}

	/**
	 * Performs server side validations for the service that has to be invoked
	 * 
	 * @param requestContainer
	 * @param responseContainer
	 * @return true if the service (action or page) has to be invoked, false if
	 *         some validation error was produced
	 */
	public boolean validate(final String businessType,
			final String businessName, RequestContextIFace requestContext) {

        Monitor monitor = null;
        
        try {
            monitor = MonitorFactory.start("validation." + businessType.toLowerCase() + "." + businessName.toLowerCase());
                        
            // Retrieve the source bean with the validations associated to busineess Name and Business Type
            // init(businessType, businessName);
            SourceBean serviceValidations = getServiceValidationSourceBean(businessName, businessType);
            
            if (serviceValidations == null){
                
                TracerSingleton
                        .log(
                                Constants.NOME_MODULO,
                                TracerSingleton.INFORMATION,
                                "Validation::validate:validazione non richiesta, nessuna busta <SERVICE> associata a "
                                        + businessName + " di tipo " + businessType);
                return true;
            }
            
            // In the service validation find the validation that satisfy the conditions
            List validationBlocks = findValidation(serviceValidations, requestContext);

            if ((validationBlocks == null) || (validationBlocks.size() == 0)) {
                    TracerSingleton
                            .log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
                                    "Validation::validate:validazione non eseguita, nessuna condizione soddisfatta");
                return true;
            }
            
            boolean validationSucceeds = true;
            for (Iterator it = validationBlocks.iterator(); it.hasNext(); ) {
                SourceBean validationBean = (SourceBean) it.next();
                String isBlocking = (String) validationBean.getAttribute("blocking");
                
                if (isBlocking == null){
                    this.blocking = true;
                }else{
                    this.blocking = Boolean.valueOf(isBlocking).booleanValue();
                }
                
                // Upper/lower case the fields, trim them, etc etc
                structure(requestContext.getServiceRequest(), validationBean);
                
                if (!validateService(requestContext, validationBean))
                    validationSucceeds = false;
            }
            return validationSucceeds;
        }
        finally {
            if (monitor != null)
                monitor.stop();
        }
		
	}//public boolean validate(final String businessType,
	
	/**
	 * Check which validation block has to be used for validation, checking the
     * conditions for each block.
	 * 
	 * @param SourceBean serviceValidations  The envelope of service validation related to a service
	 * @param requestContainer The request container
	 * @param responseContainer The response container
	 * @return The list of source bean with the validation subelement that match conditions
	 */
	public List findValidation(final SourceBean serviceValidations,
                               final RequestContextIFace requestContext) {
		
		// Get all the validation section related to the service validations source bean
		List validations = serviceValidations
				.getAttributeAsList(ValidatorLocator.VALIDATION);
		
		if (validations.size() == 0) {
			TracerSingleton
					.log(
							Constants.NOME_MODULO,
							TracerSingleton.INFORMATION,
							"Validation::validate:validazione non richiesta, nessuna busta <VALIDATION> associata a "
									+ serviceValidations.getAttribute("NAME"));
			return null;
		}
		
        List result = new ArrayList();
		
		// This loop put in the validationSourceBean the valiadtion section that satisfy 
		// condition
		for (int i = 0; i < validations.size(); i++) {
		
            boolean conditionVerified = false;
            SourceBean validationSourceBean = (SourceBean) validations.get(i);
            List conditions = validationSourceBean.getAttributeAsList("CONDITIONS.PARAMETER");
			if ((conditions == null) || (conditions.size() == 0)) {
				TracerSingleton.log(Constants.NOME_MODULO,
						TracerSingleton.DEBUG,
						"Validation::validate: nessuna condizione impostata");
                // If there are no conditions, the validation block is considered valid
                conditionVerified = true;
			} else {
                conditionVerified = true;
                for (int j = 0; j < conditions.size(); j++) {
                    SourceBean condition = (SourceBean) conditions.get(j);
                    String parameterValue = (String) condition
                            .getAttribute("VALUE");
                    String inParameterValue = null;
                    Object parameterValueObject = ContextScooping
                            .getScopedParameter(requestContext, condition);
                    if (parameterValueObject != null)
                        inParameterValue = parameterValueObject.toString();
                    if (parameterValue.equalsIgnoreCase("AF_DEFINED")) {
                        if (inParameterValue == null) {
                            conditionVerified = false;
                            break;
                        } // if (inParameterValue == null)
                        continue;
                    } // if (parameterValue.equalsIgnoreCase("AF_DEFINED"))
                    if (parameterValue.equalsIgnoreCase("AF_NOT_DEFINED")) {
                        if (inParameterValue != null) {
                            conditionVerified = false;
                            break;
                        } // if (inParameterValue != null)
                        continue;
                    } // if (parameterValue.equalsIgnoreCase("AF_NOT_DEFINED"))
                    if (!parameterValue.equalsIgnoreCase(inParameterValue)) {
                        conditionVerified = false;
                        break;
                    } // if (!(parameterValue.equalsIgnoreCase(inParameterValue)))
                } // for (int j = 0; conditions.size(); j++)
            }
                
			
			if (conditionVerified) {
                result.add(validationSourceBean);
			}
		}// for (int i =0; i < validations.size(); i++)
		
		return result;
	}//
		

	private void structure(SourceBean serviceRequest, final SourceBean validationSourceBean) {
		try {
			// Reperisco l'elenco di tutti gli attributi che mi aspetto di
			// trovare nella richiesta
			List fields = validationSourceBean
					.getAttributeAsList("FIELDS.FIELD");

			for (Iterator iter = fields.iterator(); iter.hasNext();) {
				SourceBean field = (SourceBean) iter.next();
				String fieldName = (String) field.getAttribute(NAME_ATTRIBUTE);

				String value = null;
				List values = null;
				boolean multiValue = false;
				
				if (fieldName.indexOf(LIST_SYMBOL) > -1) {
					multiValue = true;
					values = new ArrayList();
					int i = 1;

					// Verificare
					String absoluteName = fieldName.substring(0, fieldName
							.indexOf(LIST_SYMBOL));

					String name = absoluteName + i;
					String paramNumeroTotale = absoluteName + "N";

					String valNumeroTotale = (String) serviceRequest
							.getAttribute(paramNumeroTotale);
					int numeroTotale = 0;
					try {
						numeroTotale = Integer.parseInt(valNumeroTotale);
					} catch (NumberFormatException ex) {
						TracerSingleton.log(Constants.NOME_MODULO,
								TracerSingleton.CRITICAL,
								"Validation::structure:", ex);
					}

					while ((numeroTotale == 0 && serviceRequest
							.getAttribute(name) != null)
							|| i <= numeroTotale) {
						values.add(serviceRequest.getAttribute(name));
						i++;
						name = absoluteName + i;
					}
				}
				
				if (values == null) {
					// Reperisco i valori del campo dal form di input
					try {
						value = (String) serviceRequest.getAttribute(fieldName);
					} catch (ClassCastException ex) {
						values = serviceRequest.getAttributeAsList(fieldName);
					}
				}

				if (values != null || value != null) {
					if (value != null) {
						TracerSingleton.log(Constants.NOME_MODULO,
								TracerSingleton.INFORMATION, "Il campo "
										+ fieldName + " vale: " + value);
					}//if (value != null) {

					if (values != null) {
						TracerSingleton.log(Constants.NOME_MODULO,
								TracerSingleton.INFORMATION, "Il campo "
										+ fieldName + " vale: "
										+ values.toString());
					}//if (values != null) {

					String trim = (String) field.getAttribute(TRIM_ATTRIBUTE);
					String toUppercase = (String) field
							.getAttribute(TOUPPERCASE_ATTRIBUTE);

					if ((trim != null) && trim.equalsIgnoreCase("true")) {

						if (value != null)
							value = value.trim();

						for (int i = 0; values != null && i < values.size(); i++) {
							if (values.get(i) != null)
								values.set(i, ((String) values.get(i)).trim());
						}//for (int i = 0; values != null && i < values.size(); i++) {
					}//if ((trim != null) && trim.equalsIgnoreCase("true")) {

					
					if ((toUppercase != null)
							&& toUppercase.equalsIgnoreCase("true")) {

						if (value != null)
							value = value.toUpperCase();

						for (int i = 0; values != null && i < values.size(); i++) {
							if (values.get(i) != null)
								values.set(i, ((String) values.get(i))
										.toUpperCase());
						}//for (int i = 0; values != null && i < values.size(); i++) {
					}//if ((toUppercase != null)

					if (value != null)
						serviceRequest.updAttribute(fieldName, value);
					else if (values != null) {
						if (!multiValue)
							serviceRequest.updAttribute(fieldName, values);
						else {
							String localFieldName = null;
							String absoluteName = fieldName.substring(0,
									fieldName.indexOf(LIST_SYMBOL));
							for (int i = 0; i < values.size(); i++) {
								if (values.get(i) != null) {
									localFieldName = absoluteName
											+ String.valueOf(i + 1);
									serviceRequest.updAttribute(localFieldName,
											values.get(i));
								}//if (values.get(i) != null) {
							}//for (int i = 0; i < values.size(); i++) {
						}//else {
					}//else if (values != null) {
				}//if (values != null || value != null) {
			}
		} catch (Exception ste) {
			TracerSingleton.log(Constants.NOME_MODULO,
					TracerSingleton.CRITICAL, "Validation::structure:", ste);
		}

	}

	public boolean isBlocking() {
		
		return blocking;
		
	}

	/**
	 * @param requestContainer
	 * @param responseContainer
	 * @return true if the service (action or page) has to be invoked, false if
	 *         some validation error was produced
	 */
	private boolean validateService(RequestContextIFace requestContext, final SourceBean validationSourceBean) {
		SourceBean serviceRequest = requestContext.getServiceRequest();
		ValidatorLocator locator = ValidatorLocator.getInstance();
		EMFErrorHandler errorHandler = requestContext.getErrorHandler();

		TracerSingleton.log(Constants.NOME_MODULO, TracerSingleton.INFORMATION,
				"Validation::validateService:automaticValidation");

		// Controlli formali automatici reperiti dal file validation.xml
		automaticValidation(requestContext, validationSourceBean);

		// Reperisco tutti i validatori da applicare alla richiesta
		Collection validators = locator.getValidators(validationSourceBean);

		for (Iterator iter = validators.iterator(); iter.hasNext();) {
			ValidatorIFace validator = (ValidatorIFace) iter.next();

			if (validator != null) {
	            if (validator instanceof RequestContextIFace) {       
					((RequestContextIFace)validator).setRequestContext(requestContext);
	            }

				if (!validator.check(serviceRequest, errorHandler)) {
					break;
				}
			}
		}

		if (!errorHandler.getErrors().isEmpty()) {
			// Se i controlli formali hanno prodotto errori, non proseguo con
			// gli altri controlli
			return false;
		}

		// Per ogni sezione istanzio la corrispondente classe di validazione e
		// la invoco
		for (Iterator iter = validators.iterator(); iter.hasNext();) {
			ValidatorIFace validator = (ValidatorIFace) iter.next();

			if (validator != null) {
	            if (validator instanceof RequestContextIFace) {       
					((RequestContextIFace)validator).setRequestContext(requestContext);
	            }

	            if (!validator.validate(serviceRequest, errorHandler)) {
					// Break the validation chain
					break;
				}
			}
		}

		if (!errorHandler.getErrors().isEmpty()) {
			// Check if some error was produced
			return false;
		}

		// Se la validazione completa non ha dato errori, produco eventuali
		// richieste
		// di conferma da parte dell'utente
		String skipConfirm = (String) serviceRequest.getAttribute(SKIP_CONFIRM);
		if (((skipConfirm == null) || (skipConfirm.length() == 0))
				&& errorHandler.getErrors().isEmpty()) {
			for (Iterator iter = validators.iterator(); iter.hasNext()
					&& errorHandler.getErrors().isEmpty();) {
				ValidatorIFace validator = (ValidatorIFace) iter.next();

				if (validator != null) {
		            if (validator instanceof RequestContextIFace) {       
						((RequestContextIFace)validator).setRequestContext(requestContext);
		            }

		            validator.confirm(serviceRequest, errorHandler);
				}
			}

			// Verifico se  stata prodotta una richiesta di conferma
			if (!errorHandler.getErrors().isEmpty()) {
				try {
					requestContext.getServiceResponse().setAttribute(
							ASK_CONFIRM, "TRUE");
				} catch (SourceBeanException ex) {
				}
			}
		}
		return true;
	}
    
    private List handleList(final String fieldName, final SourceBean serviceRequest) {
        // Multivalue field
        List values = new ArrayList();
        int i = 1;
        String name = StringUtils.replace(fieldName, LIST_SYMBOL,
                String.valueOf(i));
        String paramNumeroTotale = StringUtils.replace(fieldName,
                LIST_SYMBOL, "N");

        String valNumeroTotale = (String) serviceRequest
                .getAttribute(paramNumeroTotale);
        int numeroTotale = 0;
        try {
            numeroTotale = Integer.parseInt(valNumeroTotale);
        } catch (NumberFormatException ex) {
        }

        while ((numeroTotale == 0 && serviceRequest
                .getAttribute(name) != null)
                || i <= numeroTotale) {
            values.add(serviceRequest.getAttribute(name));
            i++;
            name = StringUtils.replace(fieldName, LIST_SYMBOL,
                    String.valueOf(i));
        }
        return values;
    }

	private void automaticValidation(RequestContextIFace requestContext, SourceBean validationSourceBean) {

        SourceBean serviceRequest = requestContext.getServiceRequest();

        // Reperisco l'elenco di tutti gli attributi che mi aspetto di trovare
		// nella richiesta
		List fields = validationSourceBean.getAttributeAsList("FIELDS.FIELD");

		for (Iterator iter = fields.iterator(); iter.hasNext();) {
			String tipoControllo = null;
			String value = null;

			try {
				SourceBean field = (SourceBean) iter.next();

				String fieldName = (String) field.getAttribute("name");
				String fieldAliasAfterValidation = (String) field.getAttribute("aliasAfterValidation");
				
				List values = null;

				if (fieldName.indexOf(LIST_SYMBOL) > -1) {
                    values = handleList(fieldName, serviceRequest);
                }

				if (values == null) {
					// Reperisco i valori del campo dal form di input
					try {
						value = (String) serviceRequest.getAttribute(fieldName);
					} catch (ClassCastException ex) {
						values = serviceRequest.getAttributeAsList(fieldName);
					}
				}

				if (values == null) {
					values = new ArrayList(1);
					values.add(value);
				}

				for (int i = 0; i < values.size(); i++) {
					try {
						value = (String) values.get(i);
						
						List attributesOfFields = field.getContainedAttributes();
						Iterator it = attributesOfFields.iterator();
						
						Map fieldValidationParams = new HashMap();
						SourceBeanAttribute sba = null;
						while (it.hasNext()){
							sba = (SourceBeanAttribute)it.next();
							fieldValidationParams.put(sba.getKey().toUpperCase(), sba.getValue());
						}
								
						tipoControllo = (String) field
								.getAttribute(TYPE_ATTRIBUTE);
                        
                        // If type is not specified assume GENERIC type
                        if (tipoControllo == null) {
                            tipoControllo = SyntaxChecker.FIELD_GENERIC;
                            TracerSingleton.log(
                                    Constants.NOME_MODULO,
                                    TracerSingleton.INFORMATION,
                                    "Validation::validate: tipo campo non specificato. Assunto tipo GENERIC");
                        }

						String localLogicalName = fieldName;
						if (localLogicalName.indexOf(LIST_SYMBOL) != -1)
							localLogicalName = StringUtils.replace(
									fieldName, LIST_SYMBOL, String
											.valueOf(i + 1));

						SyntaxChecker.check(requestContext,
											tipoControllo, 
											value,
											localLogicalName,
											fieldAliasAfterValidation,
											fieldValidationParams);
					} catch (Exception ex) {						
						fieldAliasAfterValidation = null;
						TracerSingleton.log(Constants.NOME_MODULO,
								TracerSingleton.INFORMATION, "ex ", ex);
					}
				}// for
			} catch (Exception ex) {
				TracerSingleton.log(Constants.NOME_MODULO,
						TracerSingleton.INFORMATION,
						"ValidationModule::automaticValidation", ex);

			}

		}

	}

}
