/**

    Copyright 2004, 2006 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago.

    Spago is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 2.1 of the License, or
    any later version.

    Spago is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Spago; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

**/
package it.eng.spago.validation.impl;

import it.eng.spago.base.Constants;
import it.eng.spago.base.SourceBean;
import it.eng.spago.configuration.ConfigSingleton;
import it.eng.spago.tracing.TracerSingleton;
import it.eng.spago.validation.ValidatorIFace;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

/**
* DATE            CONTRIBUTOR/DEVELOPER    NOTE
* 13-12-2004		  Butano           The Validation comes a class
* 										    
**/
/**
 * @author boccalon
 *
 */
 
public final class ValidatorLocator {

    public final static String VALIDATION_CONFIG_HEADER    = "VALIDATIONS.SERVICE";
    public final static String VALIDATOR_CONFIG_HEADER     = "VALIDATORS.VALIDATOR";
    public final static String VALIDATION                  = "VALIDATION";
    public final static String VALIDATION_SERVICE_NAME      = "NAME";
    public final static String VALIDATION_SERVICE_CLASS     = "CLASS";
    public final static String VALIDATION_SERVICE_CONFIG    = "CONFIG";

    private static ValidatorLocator instance = null;
    private HashMap validators = null; // HashMap of Collection of SourceBean (VALIDATOR envelopes)
    
    /**
     * In questo costruttore tutti i files XML di configurazione vengono letti e con le informazioni in essi 
     * contenute vengono create delle istanze di SourceBean; questo permette di accedere alle informazioni 
     * utilizzando i metodi della classe SourceBean. 
     */
    private ValidatorLocator() throws Exception {
    	validators = new HashMap();
    } // private ValidatorLocator() throws Exception

    public static ValidatorLocator getInstance() {
        if (instance == null) {
            synchronized (ValidatorLocator.class) {
                if (instance == null) {
                    try {
                        instance = new ValidatorLocator();
                    } // try
                    catch (Exception ex) {
                        ex.printStackTrace();
                    } // catch(Exception ex) try
                } // if (_instance == null)
            } // synchronized(ValidatorLocator.class)
        } // if (_instance == null)
        return instance;
    } // public static ValidatorLocator getInstance()
    
	/**
	 * Restituisce la classe di validazione il cui nome logico  fornito come parametro. 
	 * 
	 * @param validatorName Nome logico del validatore.
	 * 
	 * Esempio :
		<VALIDATORS>
		    <VALIDATOR sectionName="ANDATA" class="com.laboratorioalfa.sample.business.validation.AnagraficaValidator"/>
		    <VALIDATOR sectionName="RITORNO" class="com.laboratorioalfa.sample.business.validation.AnagraficaValidator"/>
		</VALIDATORS>
	 */
	
	public Collection getValidators(SourceBean validationStructure) {
        
        // validationStructure is composed like this sample:
        //
        //        <VALIDATION blocking="true" validators="FIRST_VALIDATOR">
        //        <CONDITIONS>                
        //            <PARAMETER name="GENERIC" scope="SERVICE_REQUEST" value="AF_DEFINED" />
        //        </CONDITIONS>               
        //        <FIELDS>
        //            <FIELD name="GENERIC" maxLength="30"  type="GENERIC" aliasAfterValidation="CampoGenerico" />
        //        </FIELDS>
        //        </VALIDATION>

        Collection validatorsList = (Collection)validators.get(validationStructure);
    	if (validatorsList == null) {

            synchronized (ValidatorLocator.class) {
            	ConfigSingleton config = ConfigSingleton.getInstance();
    			String validatorNamesAttribute = (String)validationStructure.getAttribute("VALIDATORS");
    			if( validatorNamesAttribute != null ) {
	    			// La seguente istruzione  commentata perch  compilabile solo con il JDK 1.4
					//String[] validatorNames = validatorNamesAttribute.split(",");
	    			
	    			// Con il JDK 1.3 bisogna appoggiarsi alla libreria regexp.jar che fornisce l'oggetto RE o
    				// al StringTokenizer
    				StringTokenizer stk = new StringTokenizer(validatorNamesAttribute, ",");

//					classNames = new ArrayList(validatorNames.length);
					validatorsList = new ArrayList(stk.countTokens());
					
					// Adesso tramite i nomi dei validatori, reperisco i nomi delle corrispondenti classi
//					for (int i = 0; i < validatorNames.length; i++) {
					for (; stk.hasMoreTokens(); ) {
//						String validatorName = validatorNames[i];
						String validatorName = stk.nextToken();
						SourceBean validatorInfo = (SourceBean)config.getFilteredSourceBeanAttribute(VALIDATOR_CONFIG_HEADER, "name", validatorName);
                        if (validatorInfo == null) {
                            TracerSingleton.log(
                                    Constants.NOME_MODULO,
                                    TracerSingleton.MAJOR,
                                    "ValidatorLocator::getValidators: unable to retrieve configuration for validator " + validatorName);
                        } else {
                            String className = (String)validatorInfo.getAttribute(VALIDATION_SERVICE_CLASS);
                            if (className == null) {
                                TracerSingleton.log(
                                        Constants.NOME_MODULO,
                                        TracerSingleton.MAJOR,
                                        "ValidatorLocator::getValidators: unable to retrieve class name for validator " + validatorName);
                            } else {
                                validatorsList.add(validatorInfo);
                            }
                        }
					}
					
		    		// Metto il validatore in cache
		    		validators.put(validationStructure, validatorsList);
    			}
            }// synchronized (ValidatorLocator.class)
    	}

		Collection result = null;
		
		if (validatorsList != null) {
            Object[] initArgs = new Object[0];
			result = new ArrayList();
            
			for (Iterator iter = validatorsList.iterator(); iter.hasNext();) {
                SourceBean validatorInfo = (SourceBean)iter.next();

				try {
                    SourceBean configSection = (SourceBean)validatorInfo.getAttribute(VALIDATION_SERVICE_CONFIG);
                    String className = (String)validatorInfo.getAttribute(VALIDATION_SERVICE_CLASS);
                    
                    Class validatorClass = Class.forName(className);
					Constructor validatorConstructor = validatorClass.getConstructor(new Class[0]);
                    ValidatorIFace validatorInstance = (ValidatorIFace) validatorConstructor.newInstance(initArgs);
                    if (configSection != null) {
                        validatorInstance.init(configSection);
                    }
					result.add(validatorInstance);
				} catch (ClassNotFoundException ex) {
					TracerSingleton.log(
						Constants.NOME_MODULO,
						TracerSingleton.CRITICAL,
						"ValidatorLocator::getValidator:",
						ex);
					return null;
				} catch (NoSuchMethodException ex) {
					TracerSingleton.log(
						Constants.NOME_MODULO,
						TracerSingleton.CRITICAL,
						"ValidatorLocator::getValidator:",
						ex);
					return null;
				} catch (InstantiationException ex) {
					TracerSingleton.log(
						Constants.NOME_MODULO,
						TracerSingleton.CRITICAL,
						"ValidatorLocator::getValidator:",
						ex);
					return null;
				} catch (InvocationTargetException ex) {
					TracerSingleton.log(
						Constants.NOME_MODULO,
						TracerSingleton.CRITICAL,
						"ValidatorLocator::getValidator:",
						ex);
					return null;
				} catch (IllegalAccessException ex) {
					TracerSingleton.log(
						Constants.NOME_MODULO,
						TracerSingleton.CRITICAL,
						"ValidatorLocator::getValidator:",
						ex);
					return null;
				}
			}
		}
    	
    	return result;
    }

}

